#include <sys/param.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include "stats.h"
#include "userfile.h"    

void uf_into_user_nuke(void);

char config_file[MAXPATHLEN] = "/etc/glftpd.conf";
char rootpath[MAXPATHLEN];
char datapath[MAXPATHLEN];
int section=0;
int direction = 0;

int display_num = 10;
int num_groups = 0;

char usergroup[ 80 ][ 15 ];
char usergroup_desc[ 80 ][ 40 ];


/***************************************************************************
	USAGE
***************************************************************************/
void usage(void)
{
printf("\n\
glFtpD Stats_v3.6\n\
Options:  -u [Uploads only]\n\
	  -d [Downloads only]\n\
	  -t [Today's Tops]\n\
	  -w [Week Tops]\n\
	  -m [Month Tops]\n\
	  -a [AllTime Tops]\n\
	  -T [Today Group Tops]\n\
	  -W [Week Group Tops]\n\
	  -M [Month Group Tops]\n\
	  -A [AllTime Group Tops]\n\
	  -n [Nuke Tops]\n\
	  -x [Increase top lists]\n\
	     DEFAULT is Top 10\n\
	  -r [Use Alternate Config File]\n\
	     DEFAULT: /etc/glftpd.conf\n\
	  -s # [Stat section number]\n\
	     DEFAULT: 0 (the first section)\n\
	  -h Shows this help screen.\n\
\n\
Sample:  stats -r /etc/glftpd.conf -u -w\n\
	{ Displays Top Uploaders for the Week using /etc/glftpd.conf }\n");
}
/***************************************************************************
	PRINT HEADER
***************************************************************************/
void header(char *title)
{
printf("\
                          %-s\n",title);
printf("\
------------------------------------------------------------------------\n\
 ##  Username        Tagline                    Files     Mbytes   K/sec\n\
------------------------------------------------------------------------\n");
}
/***************************************************************************
	PRINT HEADER_GRP
***************************************************************************/
void header_grp(char *title)
{
printf("\
                         %-s\n",title);
printf("\
------------------------------------------------------------------------\n\
 ##  Group       Group Description                  Files     Mbytes  M#\n\
------------------------------------------------------------------------\n");
}
/***************************************************************************
        LOAD_SYSCONFIG
        Loads data from system configuration file.
***************************************************************************/

void sysconfig( void )
{
   FILE *configfile;
   char lvalue[ 64 ];
   char rvalue[ MAXPATHLEN ];
   int  x, y;
   char work_buff[ MAXPATHLEN ];

   strncpy(work_buff, config_file, sizeof(work_buff));
 
   if ( ( configfile = fopen( work_buff, "r" ) ) == NULL ) {
                fprintf( stderr, "bad or missing config file %s.\n", config_file);
		      return;
   }

   while (fgets( work_buff, sizeof( work_buff ), configfile ) != NULL ) {
      /* Clip out comments */
      for ( x = 0; x < strlen( work_buff ); x++ )
         if ( work_buff[ x ] == '#' )
            work_buff[ x ] = '\0';

                /* Trim */
                (void)trim( work_buff );

      /* Clear out old values */
      memset( lvalue, '\0', sizeof( lvalue ) );
      memset( rvalue, '\0', sizeof( rvalue ) );

                /* The file allows spaces embedded, but clears leading & trailin                        spaces */

      /* Parse lvalue */
      y = 0;
      for ( x = 0; x < strlen( work_buff ) && work_buff[ x ] != ' '; x++ )
         if ( isprint( work_buff[ x ] ) )
            lvalue[ y++ ] = work_buff[ x ];

      /* Parse rvalue */
      y = 0; x++;
      for ( ; x < strlen( work_buff ); x++ )
         if ( isprint( work_buff[ x ] ) )
            rvalue[ y++ ] = work_buff[ x ];

      if ( strcasecmp( lvalue, "datapath" ) == 0 )
                strncpy( datapath, rvalue, sizeof( datapath ) );
      if ( strcasecmp( lvalue, "rootpath" ) == 0 )
                strncpy( rootpath, rvalue, sizeof( rootpath ) );
   }
   fclose(configfile);
   return;
}
/*-- end of load_sysconfig() ----------------------------------------------*/

/***************************************************************************
	CLEAR_USERS
***************************************************************************/

int
clear_users( void )
{
   int x = num_users;

      memset( user[ x ].username, '\0', 25 );
      user[ x ].bytes_up = 0L;
      user[ x ].bytes_down = 0U;
      user[ x ].transfers = 0;
      user[ x ].uploads = 0;
      user[ x ].downloads = 0;
      user[ x ].total_uptime = (time_t)0L;
      user[ x ].total_downtime = (time_t)0L;
      user[ x ].total_time = (time_t)0L;
      user[ x ].upspeed = 0.0;
      user[ x ].downspeed = 0.0;
      user[ x ].times_nuked = 0;
      user[ x ].bytes_nuked = 0U;
      memset(user[x].group_user, '\0', sizeof( user[x].group_user) );
      memset(uf.group_user, '\0', sizeof(uf.group_user));
      uf.bytes_nuked = 0U;
      uf.times_nuked = 0;

   return 0;
}

/*-- end of clear_users() ------------------------------------------------*/


/***************************************************************************
	QS_LIST_UP
***************************************************************************/

void 
qs_list_up( struct USER item[], int left, int right )
{
   register int i;
   register int j;
   size_t x;
   struct USER temp;
										  
	
   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_up;

   do
   {
      while ( item[ i ].bytes_up < x && i < right )
         i++;
      while ( item[ j ].bytes_up > x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_up( item, left, j );

   if ( i < right )
      qs_list_up( item, i, right );
}
/*-- end of qs_list_up() -------------------------------------------------*/


/***************************************************************************
	QS_LIST_DOWN
***************************************************************************/

void 
qs_list_down( struct USER item[], int left, int right )
{
   register int i;
   register int j;
   size_t x;
   struct USER temp;
										 
	
   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_down;

   do
   {
      while ( item[ i ].bytes_down < x && i < right )
         i++;
      while ( item[ j ].bytes_down > x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_down( item, left, j );

   if ( i < right )
      qs_list_down( item, i, right );
}
/*-- end of qs_list_down() -----------------------------------------------*/
																		
/***************************************************************************
        PRINT_NUKE
***************************************************************************/
int
print_nuke(void)
{
        int x;
        int num = 1;
        char message[BUFSIZ];

    if (num_users == 0 || user[num_users - 1 ].bytes_up == 0 )
    {
        printf("");
        return 0;
    }
printf("                             - Nuke Top -           ");
printf("\n\
------------------------------------------------------------------------\n\
 ##  User        Tagline                              # Nuked   Total MB\n\
------------------------------------------------------------------------\n");

    for (x = num_users - 1; x > num_users - display_num - 1; x-- )
    {
        if (user[x].bytes_up != 0 )
        {
        sprintf(message, "[%02d] %-11.11s %-35.35s %8.0f %8.0fMB\n",          
        num++, user[x].username, user[x].tagline, (double)user[x].bytes_up, (double)user[x].bytes_nuked);	                    
	printf("%s", message);
        }
        else return 0;
    }
    return 0;
}

/***************************************************************************
	PRINT_UP
***************************************************************************/

int 
print_up( void )
{
   int    x;
   int    num = 1;
char   numbuf1[ BUFSIZ ];
char message[MAXPATHLEN];
	
   if ( num_users == 0 || user[ num_users - 1 ].bytes_up == 0L )
   {				
	printf( "" );
      return 0;
   }

   for ( x = num_users - 1; x > num_users - display_num - 1; x-- )
   {
      if ( user[ x ].bytes_up != 0L )
      {
			commafmt( numbuf1, BUFSIZ, user[x].uploads );
			strrev( numbuf1 );
			while ( strlen( numbuf1 ) < 7 )
				strcat( numbuf1, " " );
			strrev( numbuf1 );
					  				
			sprintf( message, "[%02d] %-11.11s  %-27.27s %-6s %8.0fMB %4.0fKBs\n",
				num++, user[x].username, user[x].tagline, numbuf1,
				(double)( user[ x ].bytes_up / 1024.0 ), (double)user[ x ].upspeed );
	printf("%s",message);
      }
      else return 0;
   }

   return 0;
}
/*-- end of print_up() ---------------------------------------------------*/


/***************************************************************************
	PRINT_DOWN
***************************************************************************/

int 
print_down( void )
{
   int    x;
   int    num = 1;
	char   numbuf1[ BUFSIZ ];
	char message[MAXPATHLEN];
										  
	
   if ( num_users == 0 || user[ num_users - 1 ].bytes_down == 0L )
   {
	printf( "" );
      return 0;
   }

   for ( x = num_users - 1; x > num_users - display_num - 1; x-- )
   {
      if ( user[ x ].bytes_down != 0L )
      {
			commafmt( numbuf1, BUFSIZ, user[x].downloads );
			strrev( numbuf1 );
			while ( strlen( numbuf1 ) < 7 )
				strcat( numbuf1, " " );
			strrev( numbuf1 );
					  				
			
			sprintf( message, "[%02d] %-11.11s  %-27.27s %6s %8.0fMB %4.0fKBs\n",
				num++, user[x].username, user[x].tagline, numbuf1, 
				(double)( user[ x ].bytes_down / 1024.0 ), (double)user[ x ].downspeed );
	printf("%s",message);
      }
      else return 0;
   }

   return 0;
}
/*-- end of print_down() --------------------------------------------------*/
										 

/***************************************************************************
	UF_INTO_USER
***************************************************************************/

void 
uf_into_user( void )
{											 

	if ( num_users >= NUM_USERS )
		return;

	memcpy( user[ num_users ].username, uf.name, 24 );
	memcpy( user[ num_users ].tagline, uf.tagline, 63 );
	memcpy( user[ num_users ].group_user, uf.group_user[0], 15);

	user[ num_users ].bytes_up = uf.bytes_up;
	user[ num_users ].bytes_down = uf.bytes_down;

	user[ num_users ].transfers = uf.files_up + uf.files_down;
	user[ num_users ].uploads = uf.files_up;
	user[ num_users ].downloads = uf.files_down;
	user[ num_users ].total_time = uf.seconds_up + uf.seconds_down;
	user[ num_users ].total_uptime = uf.seconds_up;
	user[ num_users ].total_downtime = uf.seconds_down;

   if ( user[ num_users ].total_uptime != 0L )
   	user[ num_users ].upspeed = (double)((double)(user[ num_users ].bytes_up) / (double)(user[ num_users ].total_uptime));
   if ( user[ num_users ].total_downtime != 0L )
      user[ num_users ].downspeed = (double)((double)(user[ num_users ].bytes_down) / (double)(user[ num_users ].total_downtime));

	num_users++;

   return;
}
/*-- end of uf_into_user() -----------------------------------------------*/


/***************************************************************************
	UF_INTO_USER_WK
***************************************************************************/

void 
uf_into_user_wk( void )
{											

	if ( num_users >= NUM_USERS )
		return;

	memcpy( user[ num_users ].username, uf.name, sizeof( user[num_users].username ) );
	memcpy( user[ num_users ].tagline, uf.tagline, sizeof( user[num_users].tagline ) );
	memcpy( user[ num_users ].group_user, uf.group_user[0], sizeof( user[num_users].group_user) );


	user[ num_users ].bytes_up = uf.bytes_up_wk;
	user[ num_users ].bytes_down = uf.bytes_down_wk;
									  
	
		
	user[ num_users ].transfers = uf.files_up_wk + uf.files_down_wk;
	user[ num_users ].uploads = uf.files_up_wk;
	user[ num_users ].downloads = uf.files_down_wk;
	user[ num_users ].total_time = uf.seconds_up_wk + uf.seconds_down_wk;
	user[ num_users ].total_uptime = uf.seconds_up_wk;
	user[ num_users ].total_downtime = uf.seconds_down_wk;
									  
	
   if ( user[ num_users ].total_uptime != 0L )
   	user[ num_users ].upspeed = (double)((double)(user[ num_users ].bytes_up) / (double)(user[ num_users ].total_uptime));
   if ( user[ num_users ].total_downtime != 0L )
      user[ num_users ].downspeed = (double)((double)(user[ num_users ].bytes_down) / (double)(user[ num_users ].total_downtime));

	num_users++;
									  
	
   return;
}
/*-- end of uf_into_user_wk() --------------------------------------------*/

/***************************************************************************
	UF_INTO_USER_MONTH
***************************************************************************/

void 
uf_into_user_month( void )
{											

	if ( num_users >= NUM_USERS )
		return;


	memcpy( user[ num_users ].username, uf.name, sizeof( user[num_users].username) );
	memcpy( user[ num_users ].tagline, uf.tagline, sizeof( user[num_users].tagline ) );
	memcpy( user[ num_users ].group_user, uf.group_user[0], sizeof( user[num_users].group_user) );

	user[ num_users ].bytes_up = uf.bytes_up_month;
	user[ num_users ].bytes_down = uf.bytes_down_month;
									  
		
	user[ num_users ].transfers = uf.files_up_month + uf.files_down_month	;
	user[ num_users ].uploads = uf.files_up_month;
	user[ num_users ].downloads = uf.files_down_month;
	user[ num_users ].total_time = uf.seconds_up_month + uf.seconds_down_month;
	user[ num_users ].total_uptime = uf.seconds_up_month;
	user[ num_users ].total_downtime = uf.seconds_down_month;
									  
	
   if ( user[ num_users ].total_uptime != 0L )
   	user[ num_users ].upspeed = (double)((double)(user[ num_users ].bytes_up) / (double)(user[ num_users ].total_uptime));
   if ( user[ num_users ].total_downtime != 0L )
      user[ num_users ].downspeed = (double)((double)(user[ num_users ].bytes_down) / (double)(user[ num_users ].total_downtime));

	num_users++;
									  
	
   return;
}

/***************************************************************************
	UF_INTO_USER_DAY
***************************************************************************/

void 
uf_into_user_day( void )
{											

	if ( num_users >= NUM_USERS )
		return;


	memcpy( user[ num_users ].username, uf.name, sizeof( user[num_users].username) );
	memcpy( user[ num_users ].tagline, uf.tagline, sizeof( user[num_users].tagline ) );
	memcpy( user[ num_users ].group_user, uf.group_user[0], sizeof(user[num_users].group_user) );

	user[ num_users ].bytes_up = uf.bytes_up_day;
	user[ num_users ].bytes_down = uf.bytes_down_day;
									  
		
	user[ num_users ].transfers = uf.files_up_day + uf.files_down_day;
	user[ num_users ].uploads = uf.files_up_day;
	user[ num_users ].downloads = uf.files_down_day;
	user[ num_users ].total_time = uf.seconds_up_day + uf.seconds_down_day;
	user[ num_users ].total_uptime = uf.seconds_up_day;
	user[ num_users ].total_downtime = uf.seconds_down_day;
									  
	
   if ( user[ num_users ].total_uptime != 0L )
   	user[ num_users ].upspeed = (double)((double)(user[ num_users ].bytes_up) / (double)(user[ num_users ].total_uptime));
   if ( user[ num_users ].total_downtime != 0L )
      user[ num_users ].downspeed = (double)((double)(user[ num_users ].bytes_down) / (double)(user[ num_users ].total_downtime));

	num_users++;
									  
	
   return;
}
/*-- end of uf_into_user_day() --------------------------------------------*/


/***************************************************************************
	BUILD_INFO
***************************************************************************/

int 
build_info( int method )
{
	DIR    *dirf;
	struct dirent *dn;
	struct stat st;
	char   temppath[ MAXPATHLEN + 1 ];
	
		            
   snprintf( temppath, sizeof(temppath), "%s/%s/users",rootpath, datapath );
	dirf = opendir( temppath );
	if ( !dirf )
	{
		fprintf( stderr, "error reading dir \"%s\": %s\n", temppath, strerror( errno ) );
		return 1;
	}

   clear_users();

   while ( 1 )
   {
		dn = readdir( dirf );
		if ( !dn )
			break;		  
			
	snprintf( temppath, sizeof(temppath), "%s/%s/users/%s", rootpath, datapath, dn->d_name );
	stat( temppath, &st );
		
		if ( temppath[ strlen( temppath ) ] == '~' )
			continue;
		
		if ( S_ISREG (st.st_mode) && st.st_uid != 99 && st.st_gid != 99 ) {
		  if (load_userfile(dn->d_name) != 0)
			continue;
		  switch (method) {
		   case 1:
		   	uf_into_user();
			break;
		   case 2:
		   	uf_into_user_wk();
			break;
		   case 3:
			uf_into_user_day();
			break;
		   case 4:
			uf_into_user_month();
			break;
		   case 5:
			uf_into_user_nuke();
			break;
		  }
		}
   }

  	(void)closedir( dirf );	

   return 0;
}
/*---------- end of build_info() -----------------------------------------*/


/**************************************************************************
        UF_INTO_USER_NUKE
****************************************************************************/

void
uf_into_user_nuke(void)
{
        if (num_users >= NUM_USERS )
                return;

        memcpy( user[num_users].username, uf.name, sizeof(user[num_users].username) );
	memcpy( user[num_users].tagline, uf.tagline, sizeof(user[num_users].tagline) );
	memcpy( user[num_users].group_user, uf.group_user[0], sizeof(user[num_users].group_user) );        
	user[num_users].bytes_up = uf.times_nuked;
        user[num_users].bytes_nuked = (double)uf.bytes_nuked;

        num_users++;

        return;
}
/*---------end of UF_INTO_USER_NUKE----------------------------------------*/

						  
/***************************************************************************
	LOAD_USERGROUPS
	Loads group data from "<datapath>/groups.user"
***************************************************************************/

void 
load_usergroups( void )
{
	char *comment;
  	char *tokptr;
	FILE *groupfile;
   	int  x;
	char linebuf[ BUFSIZ ];
	char fname_buff[ MAXPATHLEN+1 ];
	
	/*
	 * Clear groups:
	 */
	for ( x = 0; x < 80; x++ )
		bzero( usergroup[ x ], sizeof( usergroup[ x ] ) ); 
				  
	/*
	 * Open the groupfile
	 */		 
   snprintf( fname_buff, sizeof(fname_buff), "%s/etc/group", rootpath);
   if ( ( groupfile = fopen( fname_buff, "r" ) ) == NULL )
	{									
      return;
	}
									
	/*
	 * Read lines one at a time and fill in struct
	 */
   while ( 1 )
   {
      if ( fgets( linebuf, sizeof( linebuf ), groupfile ) == NULL )
      {				 
         fclose( groupfile );
         return;
      }
				 						  
		/*
		 * Clip out comments, they'll eventually be lost anyway :)
		 */
		if ( ( comment = strchr( linebuf, '#' ) ) != NULL )
			*comment = '\0';

		/*
		 * Trim the buffer 
		 */					
		(void)trim( linebuf ); 

/*	Why was this here ?!?!? is did cause probs with non alpha groupnames
		if ( !isalpha( linebuf[ 0 ] ) )
			continue; */
		for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ ); 

  		tokptr = strtok (linebuf, ":");
		if ( tokptr )
			strncpy( usergroup[x], tokptr, sizeof( usergroup[x] ) );
		else
			continue;

		memset(usergroup_desc[x], '\0', sizeof(usergroup_desc[x]));
		if (tokptr[strlen(tokptr)+1] != ':') {
			tokptr = strtok(NULL, ":");
			if (tokptr)
				strncpy(usergroup_desc[x], tokptr, sizeof(usergroup_desc[x]));
		}
   }
   return;
}
/*-- end of load_usergroups() --------------------------------------------*/
						

/****************************************************************************
    CLEAR_GROUP
	 Clears group stats in a given array slot.
****************************************************************************/
										
void
clear_group( int which )
{
	bzero( group[which].name, sizeof( group[which].name ) );
	group[which].members = 0;
	group[which].files_up = 0;
	group[which].bytes_up = 0UL;
	group[which].files_up_wk = 0;
	group[which].bytes_up_wk = 0UL;
        group[which].files_up_month = 0;
        group[which].bytes_up_month = 0UL;

        group[which].files_down = 0;
        group[which].bytes_down = 0UL;
        group[which].files_down_wk = 0;
        group[which].bytes_down_wk = 0UL;
        group[which].files_down_month = 0;
        group[which].bytes_down_month = 0UL;

						  
	return;
}
/*--- end of clear_group() -----------------------------------------------*/



/****************************************************************************
	UPDATE_GROUP
	Looks at uf.group_user[0] and adds the current values in 'uf' to the 
	group array.
****************************************************************************/

void update_group( void ) {
	int ct;

	for ( ct = 0; ct <= num_groups; ct++ ) {
		if (ct == num_groups) {
			/* Add a new group */
			clear_group(ct);
			
			strcpy( group[ct].name, uf.group_user[0] );
			group[ct].members = 1;
			group[ct].files_up = uf.files_up;
			group[ct].bytes_up = uf.bytes_up;
                        group[ct].files_down = uf.files_down;
                        group[ct].bytes_down = uf.bytes_down;              
                        group[ct].files_up_day = uf.files_up_day;
                        group[ct].bytes_up_day = uf.bytes_up_day;
                        group[ct].files_down_day = uf.files_down_day;
                        group[ct].bytes_down_day = uf.bytes_down_day;
	                group[ct].files_up_month = uf.files_up_month;
                        group[ct].bytes_up_month = uf.bytes_up_month;
                        group[ct].files_down_month = uf.files_down_month;                                
			group[ct].bytes_down_month = uf.bytes_down_month;
			group[ct].files_up_wk = uf.files_up_wk;
			group[ct].bytes_up_wk = uf.bytes_up_wk;
        		group[ct].files_down_wk = uf.files_down_wk;
		        group[ct].bytes_down_wk = uf.bytes_down_wk;
			num_groups++;
			return;
		} else if (!strcasecmp(group[ct].name, uf.group_user[0])) {
			/* Update this group */
			group[ct].members++;
			group[ct].files_up += uf.files_up;
			group[ct].bytes_up += uf.bytes_up;
                	group[ct].files_down += uf.files_down;
                	group[ct].bytes_down += uf.bytes_down;	
			group[ct].files_up_wk += uf.files_up_wk;
			group[ct].bytes_up_wk += uf.bytes_up_wk;
                 	group[ct].files_down_wk += uf.files_down_wk;
                 	group[ct].bytes_down_wk += uf.bytes_down_wk;		
                        group[ct].files_up_day += uf.files_up_day;
                        group[ct].bytes_up_day += uf.bytes_up_day;
                        group[ct].files_down_day += uf.files_down_day;
                        group[ct].bytes_down_day += uf.bytes_down_day;                                       
                        group[ct].files_up_month += uf.files_up_month;
                        group[ct].bytes_up_month += uf.bytes_up_month;
                        group[ct].files_down_month += uf.files_down_month;                         
			group[ct].bytes_down_month += uf.bytes_down_month;
			return;
		}
	}
}
/*-- end of update_group() -----------------------------------------------*/

/***************************************************************************
        QS_LIST_GDN_DAY
***************************************************************************/

void
qs_list_gdn_day( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;

   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_down_day;

   do
   {
      while ( item[ i ].bytes_down_day > x && i < right )
         i++;
      while ( item[ j ].bytes_down_day < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gdn_day( item, left, j );

   if ( i < right )
      qs_list_gdn_day( item, i, right );
}
/*-- end of qs_list_gdn_day() ------------------------------------------*/

/***************************************************************************
        QS_LIST_GDN_MONTH
***************************************************************************/

void
qs_list_gdn_month( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;

   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_down_month;

   do
   {
      while ( item[ i ].bytes_down_month > x && i < right )
         i++;
      while ( item[ j ].bytes_down_month < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gdn_month( item, left, j );

   if ( i < right )
      qs_list_gdn_month( item, i, right );
}
/*-- end of qs_list_gdn_month() ------------------------------------------*/

/***************************************************************************
        QS_LIST_GDN_ALLTIME
***************************************************************************/

void
qs_list_gdn_alltime( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;

   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_down;

   do
   {
      while ( item[ i ].bytes_down > x && i < right )
         i++;
      while ( item[ j ].bytes_down < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gdn_alltime( item, left, j );

   if ( i < right )
      qs_list_gdn_alltime( item, i, right );
}
/*-- end of qs_list_gdn_alltime() ------------------------------------------*/

/***************************************************************************
        QS_LIST_GDN_WEEKLY
***************************************************************************/

void
qs_list_gdn_weekly( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;

   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_down_wk;

   do
   {
      while ( item[ i ].bytes_down_wk > x && i < right )
         i++;
      while ( item[ j ].bytes_down_wk < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;

         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gdn_weekly( item, left, j );

   if ( i < right )
      qs_list_gdn_weekly( item, i, right );
}
/*-- end of qs_list_gdn_weekly---------------------------------------------*/

/***************************************************************************
	QS_LIST_GUP_DAY
***************************************************************************/

void 
qs_list_gup_day( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;
										  
   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_up_day;

   do
   {
      while ( item[ i ].bytes_up_day > x && i < right )
         i++;
      while ( item[ j ].bytes_up_day < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gup_day( item, left, j );

   if ( i < right )
      qs_list_gup_day( item, i, right );
}
/*-- end of qs_list_gup_day() ------------------------------------------*/
/***************************************************************************
	QS_LIST_GUP_MONTH
***************************************************************************/

void 
qs_list_gup_month( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;
										  
   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_up_month;

   do
   {
      while ( item[ i ].bytes_up_month > x && i < right )
         i++;
      while ( item[ j ].bytes_up_month < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gup_month( item, left, j );

   if ( i < right )
      qs_list_gup_month( item, i, right );
}
/*-- end of qs_list_gup_month() ------------------------------------------*/
											 	 
/***************************************************************************
	QS_LIST_GUP_ALLTIME
***************************************************************************/

void 
qs_list_gup_alltime( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;
										  
   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_up;

   do
   {
      while ( item[ i ].bytes_up > x && i < right )
         i++;
      while ( item[ j ].bytes_up < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gup_alltime( item, left, j );

   if ( i < right )
      qs_list_gup_alltime( item, i, right );
}
/*-- end of qs_list_gup_alltime() ------------------------------------------*/
											 	 


/***************************************************************************
	QS_LIST_GUP_WEEKLY
***************************************************************************/

void 
qs_list_gup_weekly( struct GROUP item[], int left, int right )
{
   register int i;
   register int j;
   double x;
   struct GROUP temp;
										  
   i = left;
   j = right;
   x = item[ ( left + right ) / 2 ].bytes_up_wk;

   do
   {
      while ( item[ i ].bytes_up_wk > x && i < right )
         i++;
      while ( item[ j ].bytes_up_wk < x && j > left )
         j--;
      if ( i <= j )
      {
         temp = item[ i ];
         item[ i ] = item[ j ];
         item[ j ] = temp;
         i++;
         j--;
      }
   } while ( i <= j );

   if ( left < j )
      qs_list_gup_weekly( item, left, j );

   if ( i < right )
      qs_list_gup_weekly( item, i, right );
}
/*-- end of qs_list_gup_weekly() -------------------------------------------*/
		  
/****************************************************************************
        OUTPUT_GROUPDOWN_DAY
****************************************************************************/

void
output_groupdown_day( void )
{
        char fp[MAXPATHLEN];
        int  ct;
        int  x;
        int  found = -1;


        for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
        {
                for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
                {
                        if ( !strcasecmp( group[ct].name, usergroup[x] ) )
                        {
                                found = x;
                                break;
                        }
                }

		if (group[ct].files_down_day > 0) {
        	        if ( found < 0 ) {
                	        sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n", 
                	          ct+1, group[ct].name,"",group[ct].files_down_day,
                		  (double)( group[ct].bytes_down_day / 1024.0 ), group[ct].members);                  
	                 } else {
                	        sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n",
                		  ct+1, usergroup[found],usergroup_desc[found],group[ct].files_down_day,                  
				  (double)( group[ct].bytes_down_day / 1024.0 ), group[ct].members);                  
               		 }
			printf("%s",fp);
		}
        	found = -1;
        }
}
/*-- end of output_groupdown_day() -----------------------------------*/
/****************************************************************************
        OUTPUT_GROUPDOWN_MONTH
****************************************************************************/

void
output_groupdown_month( void )
{
        char fp[MAXPATHLEN];
        int  ct;
        int  x;
        int  found = -1;


        for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
        {
                for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
                {
                        if ( !strcasecmp( group[ct].name, usergroup[x] ) )
                        {
                                found = x;
                                break;
                        }
                }

		if (group[ct].files_down_month > 0) {
	                if ( found < 0 ) {
        	                sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n", 
        	                 ct+1, group[ct].name,"",group[ct].files_down_month,
        		         (double)( group[ct].bytes_down_month / 1024.0 ), group[ct].members);                  
	                 } else {
       		                 sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n",
        		          ct+1, usergroup[found],usergroup_desc[found],group[ct].files_down_month,                  
				  (double)( group[ct].bytes_down_month / 1024.0 ), group[ct].members);                  
               		 }
			printf("%s",fp);
		}
        	found = -1;
        }
}
/*-- end of output_groupdown_month() -----------------------------------*/
/****************************************************************************
        OUTPUT_GROUPDOWN_ALLTIME
****************************************************************************/

void
output_groupdown_alltime( void )
{
        char fp[MAXPATHLEN];
        int  ct;
        int  x;
        int  found = -1;


        for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
        {
                for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
                {
                        if ( !strcasecmp( group[ct].name, usergroup[x] ) )
                        {
                                found = x;
                                break;
                        }
                }

		if (group[ct].files_down > 0) {
	                if ( found < 0 ) {
        	                sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n", 
                	         ct+1, group[ct].name,"",group[ct].files_down,
                		 (double)( group[ct].bytes_down / 1024.0 ), group[ct].members);                  
	                 } else {
        	                sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n",
        		         ct+1, usergroup[found],usergroup_desc[found],group[ct].files_down,                  
				 (double)( group[ct].bytes_down / 1024.0 ), group[ct].members);                  
                	}
			printf("%s",fp);
		}
        	found = -1;
        }
}
/*-- end of output_groupdown_alltime() -----------------------------------*/
/****************************************************************************
        OUTPUT_GROUPDOWN_WEEKLY
        Outputs group tops list (to /usr/.glFtp/grouptops.weekly)
****************************************************************************/

void
output_groupdown_weekly( void )
{
        char fp[MAXPATHLEN];
        int  ct;
        int x;
        int found = -1;



        for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
        {
                for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ ) {
                        if ( !strcasecmp( group[ct].name, usergroup[x] ) )
                        {
                                found = x;
                                break;
                        }
                }

		if (group[ct].files_down_wk > 0) {
	                if ( found < 0 ) {
        	                sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n",
        	                 ct+1, group[ct].name,"",group[ct].files_down_wk,
        		         (double)( group[ct].bytes_down_wk / 1024.0 ), group[ct].members);                      
	                } else {
	                        sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d\n",
		                 ct+1, usergroup[found],usergroup_desc[found],group[ct].files_down_wk,                  
				 (double)( group[ct].bytes_down_wk / 1024.0 ), group[ct].members);                  
                 	}
			printf("%s",fp);
		}
        	found = -1;
        }
 }
/*-- end of output_groupdown_weekly() ------------------------------------*/




/****************************************************************************
	OUTPUT_GROUPTOPS_DAY
****************************************************************************/
			
void
output_grouptops_day (void )
{
	int  ct;
	int  x; 
	int  found = -1;
	char fp[MAXPATHLEN];

							 
	for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
	{
		for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
		{
			if ( !strcasecmp( group[ct].name, usergroup[x] ) )
			{
				found = x;
				break;
			}
		}

		if (group[ct].files_up_day > 0) {
			if ( found < 0 ) {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
        		         ct+1, group[ct].name,"",group[ct].files_up_day,
        	       		 (double)( group[ct].bytes_up_day / 1024.0 ), group[ct].members);
			} else {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
				 ct+1, usergroup[found],usergroup_desc[found],group[ct].files_up_day,
				 (double)( group[ct].bytes_up_day / 1024.0 ), group[ct].members);
			}
			printf("%s",fp);
		}
		found = -1;
	}
}									 
/*-- end of output_grouptops_day() -----------------------------------*/
/****************************************************************************
	OUTPUT_GROUPTOPS_month
****************************************************************************/
			
void
output_grouptops_month (void )
{
	int  ct;
	int  x; 
	int  found = -1;
	char fp[MAXPATHLEN];

							 
	for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
	{
		for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
		{
			if ( !strcasecmp( group[ct].name, usergroup[x] ) )
			{
				found = x;
				break;
			}
		}

		if (group[ct].files_up_month > 0) {
			if ( found < 0 ) {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
        	        	 ct+1, group[ct].name,"",group[ct].files_up_month,
        		         (double)( group[ct].bytes_up_month / 1024.0 ), group[ct].members);
			} else {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
                		 ct+1, usergroup[found],usergroup_desc[found],group[ct].files_up_month,
		                 (double)( group[ct].bytes_up_month / 1024.0 ), group[ct].members);
			}
			printf("%s",fp);
		}
		found = -1;
	}
}									 
/*-- end of output_grouptops_month() -----------------------------------*/
/****************************************************************************
	OUTPUT_GROUPTOPS_ALLTIME
****************************************************************************/
			
void
output_grouptops_alltime( void )
{
	int  ct;
	int  x; 
	int  found = -1;
	char fp[MAXPATHLEN];

							 
	for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
	{
		for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
		{
			if ( !strcasecmp( group[ct].name, usergroup[x] ) )
			{
				found = x;
				break;
			}
		}

		if (group[ct].files_up > 0) {
			if ( found < 0 ) {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
        		         ct+1, group[ct].name,"",group[ct].files_up,
                		 (double)( group[ct].bytes_up / 1024.0 ), group[ct].members);
			} else {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
        		         ct+1, usergroup[found],usergroup_desc[found],group[ct].files_up,
        	        	 (double)( group[ct].bytes_up / 1024.0 ), group[ct].members);
			}
			printf("%s",fp);
		}
		found = -1;
	}
}									 
/*-- end of output_grouptops_alltime() -----------------------------------*/
	 

/****************************************************************************
	OUTPUT_GROUPTOPS_WEEKLY
	Outputs group tops list (to /usr/.phear/grouptops.weekly)
****************************************************************************/
			
void
output_grouptops_weekly( void )
{
	int  ct; 
	int x;
	int found = -1;
	char fp[MAXPATHLEN];

							 
	for ( ct = 0; ct < num_groups && ct < display_num; ct++ )
	{
		for ( x = 0; x < 80 && usergroup[x][0] != '\0'; x++ )
			if ( !strcasecmp( group[ct].name, usergroup[x] ) )
			{
				found = x;
				break;
			}

		if (group[ct].files_up_wk > 0) {	
			if ( found < 0 ) {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
      			         ct+1, group[ct].name,"",group[ct].files_up_wk,
         			 (double)( group[ct].bytes_up_wk / 1024.0 ), group[ct].members);
			} else {
				sprintf( fp, "[%02d] %-10.10s %-34.34s %6d %8.0fMB %3d \n",
        		         ct+1, usergroup[found],usergroup_desc[found],group[ct].files_up_wk,
                		 (double)( group[ct].bytes_up_wk / 1024.0 ), group[ct].members);
			}
			printf("%s",fp);
		}
		found = -1;
	}
}									 
/*-- end of output_grouptops_weekly() ------------------------------------*/

	

/***************************************************************************
	ACCUM_STATS
	Reads all userfiles, taking the FIRST entry for group_user and adding 
	that user's stats to the group entry.
***************************************************************************/
										
void accum_stats(void) {
	char   temppath[ MAXPATHLEN ];
  	char   username[ MAXPATHLEN ];
	char   tempbuf[ BUFSIZ ];
  	DIR    *dirf;
  	struct dirent *dn;
  	struct stat st;
	int    ct;
	
  	snprintf (temppath, sizeof(temppath), "%s/%s/users", rootpath, datapath );
  	dirf = opendir (temppath);
  	if (!dirf) {
		/* Error opening directory?? */
		fprintf( stderr, "grouptop: error opening dir '%s': %s\n", temppath, strerror( errno ) );
      		return;
   	}
	
  	while (1) {
		dn = readdir (dirf);
   		if (!dn)
			break;

      		snprintf (temppath, sizeof(temppath), "%s/%s/users/%s", rootpath, datapath, dn->d_name);
		strncpy (username, dn->d_name, sizeof (username));
		stat (temppath, &st);
		if (S_ISREG(st.st_mode) && st.st_uid != 99 && st.st_gid != 99) {
			if (!strncmp(username, "default.", 8))
				continue;
   			sprintf (tempbuf, " %-8.8s ", username);
			load_userfile( username );
			
			if ( uf.group_user[0][0] != '\0' )
			{
				/* User is in a group! */
				/* update_group knows where to get the info & where to put it */
				update_group();
			}
		}
	}

  	(void) closedir (dirf);
}
/*-- end of accum_stats() ------------------------------------------------*/


/***************************************************************************
	MAIN
***************************************************************************/
int main( int argc, char **argv )	{     
	int c;
	int end = 0;
	int match = 0;
	extern int optopt;
	extern char *optarg;
	int dailies, weeklies, monthlies, alltimes;
	int groupday, groupweeklies, groupmonth, groupalltimes;
	int nuketops;

	/* set init value's */
	dailies = 0;
	weeklies = 0;
	monthlies = 0;
	alltimes = 0;
	groupday = 0;
	groupweeklies = 0;
	groupmonth = 0;
	groupalltimes = 0;
	nuketops = 0;
	
	/* get all options */
	while ((c = getopt(argc, argv, ":r:x:s:wtmaAWnNTMuUdDhH?")) != -1) {
	    switch(c) {
		case 'r':
			strncpy(config_file, optarg, sizeof(config_file));
			config_file[sizeof(config_file)] = '\0';
			if (!match)
			    match = 1;
			break;
		case 'x':
			display_num = atoi(optarg);
			if (!match)
			    match = 1;
			break;
		case 'w':
			weeklies = 1;	
			if (!match)
			    match = 1;
			break;
		case 't':
			dailies = 1;	
			if (!match)
			    match = 1;
			break;
		case 'm':
			monthlies = 1;	
			if (!match)
			    match = 1;
			break;
		case 'a':
			alltimes = 1;
			if (!match)
			    match = 1;
			break;	
		case 'A':
			groupalltimes = 1;
			if (!match)
			    match = 1;
			break;
		case 'W':
			groupweeklies = 1;
			if (!match)
			    match = 1;
			break;
		case 'n':
		case 'N':
			nuketops = 1;
			if (!match)
			    match = 1;
			break;
		case 'T':
			groupday = 1;
			if (!match)
			    match = 1;
			break;
		case 'M':
			groupmonth = 1;
			if (!match)
			    match = 1;
			break;
		case 'u':
		case 'U':
			direction = 1;
			if (!match)
			    match = 1;
			break;
		case 'd':
		case 'D':
			direction = 2;
			if (!match)
			    match = 1;
			break;
		case 's':
			section = atoi(optarg);
			if (section > 0)
			    section = section * 3;
			if (!match)
			    match = 1;
			break;
		case 'h':
		case 'H':
		case '?':
		default:
			usage();
			end = 1;
			break;	    												    
	    }
	}

	/* check if they even specified any args */
	if (argc < 2 || !match) {
	    usage();
	    return 0;
	}    

	/* load the sysconfig */
	sysconfig();

	/* check if a correct display number is specified, ifnot put it to 10 */  
	if (display_num < 1)
	    display_num = 10;

	/* get the stats and show them */
	if (dailies)
	    Make_Dailies();
	if (weeklies)
	    Make_Weeklies();
	if (monthlies)
	    Make_Monthlies();
	if (alltimes)
	    Make_AllTimes();
	if (groupday)
	    Make_GroupDay();
	if (groupweeklies)
	    Make_GroupWeeklies();
	if (groupmonth)
	    Make_GroupMonth();
	if (groupalltimes)
	    Make_GroupAllTimes();
	if (nuketops)
	    Make_NukeTops();

    return 0;   
}

int Make_Weeklies() {
	clear_users();
	num_users = 0;
	if ( build_info(2) != 0 )	{
	   	fprintf( stderr, "stats: error building alltime list\n" );
		return 1;
	}
	if ( num_users != 0 && direction < 2) {
		qs_list_up( user, 0, num_users - 1 );
		header("Week Top (Up)");
		print_up();
	}
	if ( num_users != 0 && direction != 1 ) {
		qs_list_down( user, 0, num_users - 1 );
		header("Week Top (Down)");
		print_down();
	}
	return 0;
}

/* Daily */
int Make_Dailies() {
	clear_users();
	num_users = 0;
	if ( build_info(3) != 0 ) {
	   	fprintf( stderr, "stats: error building daily list\n" );
		return 1;
	}
	if ( num_users != 0 && direction < 2) {
		qs_list_up( user, 0, num_users - 1 );
		header("Today's Top (Up)");
		print_up();
	}
	if ( num_users != 0 && direction != 1 ) {
		qs_list_down( user, 0, num_users - 1 );
		header("Today's Top (Down)");
		print_down();
	}
	return 0;
}

/* Monthly */
int Make_Monthlies() {

	clear_users();
	num_users = 0;
	if ( build_info(4) != 0 ) {
	   	fprintf( stderr, "stats: error building monthly list\n" );
		return 1;
	}
	if ( num_users != 0 && direction < 2) {
		qs_list_up( user, 0, num_users - 1 );
		header("Month Top (Up)");
		print_up();			
	}
	if ( num_users != 0 && direction != 1) {
		qs_list_down( user, 0, num_users - 1 );
		header("Month Top (Down)");
		print_down();
	}
	return 0;
}

/* Alltime */
int Make_AllTimes() {
	clear_users();
	num_users = 0;
	if ( build_info(1) != 0 ) {
		fprintf( stderr, "stats: error building alltime list\n" );
		return 1;
	}
	if ( num_users != 0 && direction < 2) {
		qs_list_up( user, 0, num_users - 1 );
		header("All Time Top (Up)");
		print_up();
	}
	if ( num_users != 0 && direction != 1) {
		qs_list_down( user, 0, num_users - 1 );
		header("All Time Top (Down)");
		print_down();
	}
	return 0;
}

int Make_GroupDay() {
	num_groups = 0;
	load_usergroups();
	accum_stats();

    if ( num_groups != 0 && direction < 2) {
        qs_list_gup_day( group, 0, num_groups - 1 );
	header_grp("Group Today Top (Up)");
	output_grouptops_day();
    }
    if ( num_groups != 0 && direction != 1) {
        qs_list_gdn_day( group, 0, num_groups - 1 );
	header_grp("Group Today Top (Down)");
	output_groupdown_day();
    }
	return 0;
}

int Make_GroupMonth() {
	num_groups = 0;
	load_usergroups();
	accum_stats();

    if ( num_groups != 0 && direction < 2) {
        qs_list_gup_month( group, 0, num_groups - 1 );
	header_grp("Group Month Top (Up)");
	output_grouptops_month();
    }
    if ( num_groups != 0 && direction != 1) {
        qs_list_gdn_month( group, 0, num_groups - 1 );
	header_grp("Group Month Top (Down)");
	output_groupdown_month();
    }
	return 0;
}

int Make_GroupAllTimes() {
	num_groups = 0;
	load_usergroups();
	accum_stats();

    if ( num_groups != 0 && direction < 2) {
        qs_list_gup_alltime( group, 0, num_groups - 1 );
	header_grp("Group All Time Top (Up)");
	output_grouptops_alltime();
    }
    if ( num_groups != 0 && direction != 1) {
        qs_list_gdn_alltime( group, 0, num_groups - 1 );
	header_grp("Group All Time Top (Down)");
	output_groupdown_alltime();
    }
	return 0;
}

int Make_GroupWeeklies() {
	num_groups = 0;
	load_usergroups();
	accum_stats();

    if ( num_groups != 0 && direction < 2) {
        qs_list_gup_weekly( group, 0, num_groups - 1 );
	header_grp("Group Week Top (Up)");
	output_grouptops_weekly();
    }
    if ( num_groups != 0 && direction != 1) {
        qs_list_gdn_weekly( group, 0, num_groups - 1 );
	header_grp("Group Week Top (Down)");
	output_groupdown_weekly();
    }
	return 0;
}

int Make_NukeTops() {
 	num_users = 0;
	if (build_info(5)) {
		fprintf( stderr, "stats: error building nuketop list\n" );
		return 1;
	}

	if ( num_users != 0 ) {
	   qs_list_up (user, 0, num_users - 1 );
	} 
	print_nuke();
	return 0;
}
